package dev.peerat.parser.java.operation;

import java.util.List;
import java.util.function.Function;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.VisitorBag;

public class SynchronizedOperation extends OperationBag{
	
	private Value value;
	
	public SynchronizedOperation(Value value){
		super();
		this.value = value;
	}
	
	public Value getValue(){
		return this.value;
	}

	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder){
		if(finder.apply(value)) return (E)value;
		return super.find(finder);
	}
	
	@Override
	public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list){
		if(finder.apply(value)) list.add((E)value);
		value.findAll(finder, list);
		super.findAll(finder, list);
	}

	@Override
	public void build(Builder builder) throws Exception {
		
	}

	@Override
	public VisitorBag visit(JavaVisitor<?> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		bag.merge(this.value.visit(visitor));
		bag.merge(super.visit(visitor));
		return bag;
	}
}
