package dev.peerat.parser.java.tree;

import java.util.function.BiFunction;

import dev.peerat.parser.Bag;
import dev.peerat.parser.TokenType;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.value.BiValue;
import dev.peerat.parser.java.value.MethodCallValue;
import dev.peerat.parser.java.value.ModifierValue;
import dev.peerat.parser.java.value.StaticValue;
import dev.peerat.parser.java.value.TriValue;
import dev.peerat.parser.java.value.Value.ValueContainer;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class ValueTree extends SyntaxTree<JavaElement> {

	public ValueTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new StateTree<>();
	}

	@Override
	public void init(){
		StateTree<JavaElement> type = this.registery.getNode(TypeTree.class);
		StateTree<JavaElement> function_container = this.registery.getNode(FunctionContainerTree.class);
		StateTree<JavaElement> class_container = this.registery.getNode(ClassContainerTree.class);
		// VALUE
		StateTree<JavaElement> value = this.current;
		StateTree<JavaElement> unary_value = new StateTree<>();
		StateTree<JavaElement> value_operation = new StateTree<>();
		
		value.then((v) -> {
			return v.validate((t) -> {
				System.out.println("entry value "+t);
				return false;
			});
		});
		
		BiFunction<JavaElement, Bag, JavaElement> operationBuilder = (parent, bag) -> {
			System.out.println("operationBuilder "+bag);
			if (bag.has("current")) {
				bag.set(new BiValue(new StaticValue(bag.get("current")), bag.get("action"), bag.get("right")));
			} else {
				bag.set(new BiValue(bag.get(), bag.get("action"), bag.get("right")));
			}
			System.out.println("value_op " + bag);
			if(parent instanceof ValueContainer) ((ValueContainer)parent).addValue(bag.get());
			return null;
		};
		
		BiFunction<JavaElement, Bag, JavaElement> staticValueBuilder = (parent, bag) -> {
			System.out.println("staticValueBuilder "+bag);
			StaticValue result = new StaticValue(bag.get("current"));
			bag.set(result);
			if(parent instanceof ValueContainer) ((ValueContainer)parent).addValue(result);
			return null;
		};
		
		BiFunction<JavaElement, Bag, JavaElement> modifierValueBuilder = (parent, bag) -> {
			System.out.println("modifierValueBuilder "+bag);
			ModifierValue result = new ModifierValue(bag.get("modifier"), bag.get());
			bag.set(result);
			if(parent instanceof ValueContainer) ((ValueContainer)parent).addValue(result);
			return null;
		};
		
		BiFunction<JavaElement, Bag, JavaElement> methodCallValueBuilder = (parent, bag) -> {
			System.out.println("methodCallValueBuilder "+bag);
			MethodCallValue result = new MethodCallValue(null, null, bag.get("current"), null);
			bag.set(result);
			if(parent instanceof ValueContainer) ((ValueContainer)parent).addValue(result);
			return null;
		};
		
		StateTree<JavaElement> redirectRight = new RedirectStateTree<>(value, (global, local) -> {
			System.out.println("redirectRight g="+global+" ,l="+local);
			global.set("right", local.get());
		});

		StateTree<JavaElement> value_instance = unary_value.equals("new");
		StateTree<JavaElement> value_instance_type = value_instance.redirect(type);
		StateTree<JavaElement> value_instance_params = value_instance_type.equals("(");
		StateTree<JavaElement> value_instance_params_end = value_instance_params.equals(")");
		StateTree<JavaElement> value_instance_param = value_instance_params.redirect(value);
		value_instance_param.then(value_instance_params_end);
		value_instance_param.equals(",").then(value_instance_param);
		value_instance_params_end.<JavaElement>end();
		value_instance_params_end.equals("{")
		.<JavaElement>end().multiple(class_container)
		.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end();
		
		StateTree<JavaElement> value_instance_array_init = value_instance_type.equals("{");
		StateTree<JavaElement> value_instance_array_init_value = value_instance_array_init.redirect(value);
		value_instance_array_init_value.equals(",").then(value_instance_array_init_value);
		value_instance_array_init_value.equals("}").end();
		
		StateTree<JavaElement> value_instance_array = value_instance_type.equals("[");
		StateTree<JavaElement> value_instance_array_value = value_instance_array.redirect(value);
		StateTree<JavaElement> value_instance_array_end = value_instance_array_value.equals("]");
		value_instance_array.then(value_instance_array_end);
		value_instance_array_end.end();
		value_instance_array_end.then(value_instance_array);
		
		StateTree<JavaElement> value_name = unary_value
				.then((validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME),
						(bag, token) -> bag.set("current", token)));
		value_name.end(staticValueBuilder);
		value_name.then(value_operation);
		
		unary_value.then((validator) -> validator.validate(
				(token) -> token.getType().equals(TokenType.STRING) || token.getType().equals(TokenType.CHAR),
				(bag, token) -> bag.set("current", token)))
			.end(staticValueBuilder);

		value_operation.equals(">","action",">").then(redirectRight).end(operationBuilder);
		value_operation.equals("<","action","<").then(redirectRight).end(operationBuilder);
		value_operation.equals("<","action","<").equals("<","action","<<").then(redirectRight).end(operationBuilder);
		value_operation.equals("+","action","+").then(redirectRight).end(operationBuilder);
		StateTree<JavaElement> value_operation_bit_and = value_operation.equals("&","action","&");
		value_operation_bit_and.then(redirectRight).end(operationBuilder);
		value_operation_bit_and.equals("&","action","&&").then(redirectRight).end(operationBuilder);
		StateTree<JavaElement> value_operation_bit_or = value_operation.equals("|","action","|");
		value_operation_bit_or.then(redirectRight).end(operationBuilder);
		value_operation_bit_or.equals("|","action","||").then(redirectRight).end(operationBuilder);
		
		value.equals("!","modifier").redirect(value).end(modifierValueBuilder);
		
		StateTree<JavaElement> left_value = value.redirect(unary_value);
		left_value.end((parent, bag) -> {
			if(parent instanceof ValueContainer) ((ValueContainer)parent).addValue(bag.get());
			return null;
		});
		left_value.equals("?")
			.redirect(value, (global, local) -> global.set("success", local.get()))
			.equals(":")
			.redirect(value, (global, local) -> global.set("fail", local.get()))
			.end((parent, bag) -> {
				System.out.println("tri value "+bag);
				bag.set(new TriValue(bag.get(), bag.get("success"), bag.get("fail")));
				if(parent instanceof ValueContainer) ((ValueContainer)parent).addValue(bag.get());
				return null;
			});
		
		StateTree<JavaElement> value_array = left_value.equals("[");
		StateTree<JavaElement> value_array_value = value_array.redirect(value);
		StateTree<JavaElement> value_array_end = value_array_value.equals("]");
		value_array_end.end();
		value_array_end.then(value_array);
		
		StateTree<JavaElement> operation_call = value_name.equals(".");
		operation_call.then(value_name);
		StateTree<JavaElement> operation_begin = value_name.equals("(");
		StateTree<JavaElement> operation_end = operation_begin.equals(")");
		operation_end.then(operation_call);
		operation_end.end(methodCallValueBuilder);
		StateTree<JavaElement> operation_value = operation_begin.redirect(value);
		operation_value.then(operation_end);
		operation_value.equals(",")
				.then(operation_value);
		
		StateTree<JavaElement> value_parenthesis = unary_value.equals("(");
		StateTree<JavaElement> value_parenthesis_close = value_parenthesis.equals(")");
		StateTree<JavaElement> value_inside = value_parenthesis.redirect(value);
		StateTree<JavaElement> value_inside_type = value_parenthesis.redirect(type);
		
		value_inside_type.then(value_inside);
		StateTree<JavaElement> value_inside_splitter = value_inside.equals(",");
		value_inside_splitter.then(value_inside_type);
		value_inside_splitter.then(value_inside);
		
		value_inside.then(value_parenthesis_close);
		value_inside.then(value_operation);
		value_parenthesis_close.end();
		value_parenthesis_close.then(value_operation);
		
		StateTree<JavaElement> value_lambda = value_parenthesis_close.then((validator) -> validator.validate((token) -> token.getValue().equals("-")) && validator.validate((token) -> token.getValue().equals(">")));
		value_lambda.equals("{")
			.<JavaElement>end().multiple(function_container)
			.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end();
		
		value_lambda.redirect(unary_value).end();
		
	}

}
