package dev.peerat.parser.java.visitor;

import dev.peerat.parser.java.AnnotationClass;
import dev.peerat.parser.java.Import;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Operation;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public abstract class JavaVisitor<T extends JavaElement> extends Visitor<JavaElement>{
	
	public static <T extends JavaElement> JavaVisitor<T> base(Visitor<T> visitor){
		return new JavaVisitor<T>(null){
			
			@Override
			public boolean canVisit(Class<?> type){
//				System.out.println("base can visit ? "+type);
				return visitor.canVisit(type);
			}
			
			@Override
			public VisitorBag visitElement(T element){
				return visitor.visit(element);
			}
		};
	}

	public static JavaVisitor<JavaElement> allJavaElement(){
		return new JavaVisitor<JavaElement>(JavaElement.class){

			@Override
			public VisitorBag visitElement(JavaElement element){
				VisitorBag bag = new VisitorBag();
				bag.compute(element);
				return bag;
			}
			
		};
	}
	
	public static JavaClassBaseVisitor allClassBase(){
		return new JavaClassBaseVisitor();
	}
	
	public static JavaClassVisitor allClass(){
		return new JavaClassVisitor();
	}
	
	public static JavaVisitor<JavaElement> allInterface(){
		return null;
	}
	
	public static JavaVisitor<JavaElement> allEnum(){
		return null;
	}
	
	public static JavaVisitor<AnnotationClass> allAnnotationClass(){
		return null;
	}
	
	public static JavaAnnotationVisitor allAnnotation(){
		return new JavaAnnotationVisitor();
	}
	
	public static JavaVariableVisitor allVariable(){
		return new JavaVariableVisitor();
	}
	
	public static JavaFunctionVisitor allFunction(){
		return new JavaFunctionVisitor();
	}
	
	public static JavaVisitor<Value> allValue(){
		return null;
	}
	
	public static JavaVisitor<Operation> allOperation(){
		return null;
	}
	
	public static JavaVisitor<Import> allImport(){
		return null;
	}
	
	private Class<?> type;
	
	JavaVisitor(Class<?> type){
		this.type = type;
	}
	
	@Override
	public boolean canVisit(Class<?> type){
//		System.out.println("can "+type+" visit "+this.type+" ? from "+Thread.currentThread().getStackTrace()[2]);
		return this.type.isAssignableFrom(type);
	}
	
	@Override
	public VisitorBag visit(JavaElement element){
		return visitElement((T) element);
	}
	
	public abstract VisitorBag visitElement(T element);
}